//
//  ANGTCPConnection.h
//  iCompta Mobile
//
//  Created by Cyril Anger on 17/04/08.
//  Copyright 2008 Angeman. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <sys/socket.h>

@class ANGTCPServer;

@protocol ANGTCPConnectionDelegate;

@interface ANGTCPConnection : NSObject {
	CFRunLoopRef runLoop;
	CFReadStreamRef readStream;
	CFWriteStreamRef writeStream;
	struct sockaddr *localAddress;
	struct sockaddr *remoteAddress;
	NSUInteger state;
	ANGTCPServer *server;
	BOOL invalidating;
	id <ANGTCPConnectionDelegate> delegate;
}

@property (readonly) UInt16 localPort;
@property (readonly) NSString *localAddress;
@property (readonly) UInt16 remotePort;
@property (readonly) NSString *remoteAddress;
@property (assign) ANGTCPServer *server;
@property (readonly) BOOL isValid;
@property (readonly) BOOL hasDataAvailable;
@property (assign) id <ANGTCPConnectionDelegate> delegate;

- (id)initWithReadStream:(CFReadStreamRef)newReadStream writeStream:(CFWriteStreamRef)newWriteStream;
- (id)initWithSocketHandle:(int)socket;
- (id)initWithRemoteHostName:(NSString *)name port:(UInt16)port;
- (id)initWithRemoteAddress:(const struct sockaddr *)address;
- (void)invalidate;
- (void)handleStreamEvent:(CFStreamEventType)type forStream:(CFTypeRef)stream;
- (NSData *)receiveData;
- (BOOL)sendData:(NSData *)data;

@end

@protocol ANGTCPConnectionDelegate <NSObject>

@optional

- (void)connectionDidFailOpening:(ANGTCPConnection *)connection;
- (void)connectionDidOpen:(ANGTCPConnection *)connection;
- (void)connectionDidClose:(ANGTCPConnection *)connection;
- (void)connection:(ANGTCPConnection *)connection didReceiveData:(NSData *)data;

@end
