//
//  ANGObjectsManager.h
//  ANGAdditions
//
//  Created by Cyril Anger on 05/12/07.
//  Copyright 2007 Angeman. All rights reserved.
//

#import <Cocoa/Cocoa.h>

#import <ANGAdditions/ANGObject.h>

@class ANGEncryptionWindow;

@interface ANGObjectsManager : ANGObject {
	NSString *password;
	BOOL usesEncryption;
	BOOL failedDecryption;
	NSUndoManager *undoManager;
	NSMutableDictionary *requests;
	
	IBOutlet NSObjectController *managerController;
	IBOutlet ANGEncryptionWindow *encryptWindow;
	IBOutlet ANGEncryptionWindow *decryptWindow;
	IBOutlet NSSecureTextField *currentPasswordField;
	IBOutlet NSSecureTextField *newPasswordField;
	IBOutlet NSSecureTextField *confirmPasswordField;
	IBOutlet NSButton *encryptButton;
	IBOutlet NSButton *decryptButton;
}

@property (retain) NSString *password;
@property (assign) BOOL usesEncryption;
@property (assign) BOOL failedDecryption;
@property (retain) NSUndoManager *undoManager;
@property (retain) NSMutableDictionary *requests;

+ (NSString *)arrayKeyForClass:(NSString *)className;
+ (NSString *)propertyKeyForClass:(NSString *)className;
+ (id)managerWithContentsOfFile:(NSString *)file password:(NSString *)password;
+ (id)managerWithData:(NSData *)data password:(NSString *)password;

- (BOOL)writeToFile:(NSString *)file;
- (NSData *)data;
- (id)unarchiveObjectWithData:(NSData *)data;
- (NSData *)archivedDataWithRootObject:(id)object;
- (id)copyObject:(id)object;
- (id)decryptObject:(NSData *)data;
- (NSData *)encryptObject:(id)object;
- (id)objectForClass:(NSString *)className forKey:(NSString *)key;
- (void)requestObjectForClass:(NSString *)className forKey:(NSString *)key forTarget:(id)target action:(SEL)action;
- (void)performObjectRequests:(id)object;
- (void)createMissingObjects;
- (id)defaultObjectForClass:(NSString *)className;
- (NSString *)validKeyForClass:(NSString *)className forKey:(NSString *)key;
- (void)showEncryptWindowAboveWindow:(NSWindow *)window;
- (void)showDecryptWindowAboveWindow:(NSWindow *)window target:(id)target action:(SEL)action;

- (IBAction)modifyEncryption:(id)sender;

@end

@interface ANGKeyedArchiver : NSKeyedArchiver {
	BOOL isSaving;
}

@property (assign) BOOL isSaving;

@end

@interface ANGKeyedUnarchiver : NSKeyedUnarchiver {
	ANGObjectsManager *manager;
	NSString *password;
}

@property (assign) ANGObjectsManager *manager;
@property (retain) NSString *password;

@end

@interface ANGEncryptionWindow : NSWindow {
	NSMutableArray *toolbarItemStates;
}

@property (retain) NSMutableArray *toolbarItemStates;

- (void)showAboveWindow:(NSWindow *)window;
- (void)shake;

@end
